<?php
/*
Plugin Name: Redirections
Description: Both 404 and 301 redirects
Version: 1.0
Author: Zinavo
Author URI: https://www.zinavo.com/
Plugin URI: https://www.zinavo.com/redirection-manager-wordpress-plugin.html
Text Domain: redirections
*/

add_filter('plugin_row_meta', function ($links, $file) {
    if (plugin_basename(__FILE__) === $file) {
        foreach ($links as &$link) {
            if (strpos($link, 'href="https://www.zinavo.com/redirection-manager-wordpress-plugin.html"') !== false) {
                $link = '<a href="https://www.zinavo.com/redirection-manager-wordpress-plugin.html" target="_blank">View details</a>';
            }
        }
        
        // Add Documentation link
        $links[] = '<a href="https://www.zinavo.com/wp-content/uploads/2025/02/Documentation-for-Redirection-Manager_Wordpress-Plugins.pdf" target="_blank">Documentation</a>';
        
        // Add Video Tutorials link
        $links[] = '<a href="https://youtu.be/eCGBFmfO6X0" target="_blank">Video Tutorials</a>';
    }
    return $links;
}, 10, 2);


// Prevent direct access
if (!defined('ABSPATH')) {
    exit;
}

// Rest of your plugin code...

class RedirectSuite {
    private $logs_table;

    public function __construct() {
        global $wpdb;
        $this->logs_table = $wpdb->prefix . 'redirect_logs';

        // Initialize plugin
        add_action('init', array($this, 'handle_redirects'));
        add_action('template_redirect', array($this, 'handle_404'));
        add_action('admin_menu', array($this, 'add_admin_menus'));
        add_action('admin_enqueue_scripts', array($this, 'enqueue_admin_scripts'));
        register_activation_hook(__FILE__, array($this, 'create_log_table'));
    }

    public function create_log_table() {
        global $wpdb;
        
        $charset_collate = $wpdb->get_charset_collate();
        
        $sql = "CREATE TABLE IF NOT EXISTS $this->logs_table (
            id mediumint(9) NOT NULL AUTO_INCREMENT,
            source_url varchar(255) NOT NULL,
            destination_url varchar(255) NOT NULL,
            redirect_type varchar(10) NOT NULL,
            access_date datetime DEFAULT CURRENT_TIMESTAMP,
            ip_address varchar(45),
            user_agent text,
            PRIMARY KEY  (id)
        ) $charset_collate;";
        
        require_once(ABSPATH . 'wp-admin/includes/upgrade.php');
        dbDelta($sql);
    }

    public function enqueue_admin_scripts($hook) {
        if (strpos($hook, 'redirect-suite') !== false) {
            wp_enqueue_script('jquery');
        }
    }

    public function add_admin_menus() {
        // Add main menu
        add_menu_page(
            'Redirect Suite',
            'Redirect Suite',
            'manage_options',
            'redirect-suite',
            null,
            'dashicons-randomize'
        );

        // Add submenu pages
        add_submenu_page(
            'redirect-suite',
            '404 Redirects',
            '404 Redirects',
            'manage_options',
            'redirect-suite-404',
            array($this, 'render_404_page')
        );

        add_submenu_page(
            'redirect-suite',
            '301 Redirects',
            '301 Redirects',
            'manage_options',
            'redirect-suite-301',
            array($this, 'render_301_page')
        );

        // Remove duplicate submenu item
        remove_submenu_page('redirect-suite', 'redirect-suite');
    }

    public function handle_404() {
        if (is_404()) {
            $custom_404 = get_option('custom_404_page');
            if (!empty($custom_404)) {
                $this->log_redirect($_SERVER['REQUEST_URI'], $custom_404, '404');
                wp_redirect(home_url($custom_404), 302);
                exit;
            }
        }
    }

    public function handle_redirects() {
        $current_url = $_SERVER['REQUEST_URI'];
        $redirect_rules = get_option('redirect_rules', array());

        if (is_array($redirect_rules) && isset($redirect_rules[$current_url])) {
            $this->log_redirect($current_url, $redirect_rules[$current_url], '301');
            wp_redirect($redirect_rules[$current_url], 301);
            exit;
        }
    }

    private function log_redirect($source, $destination, $type) {
        global $wpdb;
        
        $wpdb->insert(
            $this->logs_table,
            array(
                'source_url' => $source,
                'destination_url' => $destination,
                'redirect_type' => $type,
                'ip_address' => $_SERVER['REMOTE_ADDR'],
                'user_agent' => $_SERVER['HTTP_USER_AGENT']
            )
        );
    }

    public function render_404_page() {
        if (isset($_POST['save_404_settings']) && check_admin_referer('404_settings_nonce')) {
            update_option('custom_404_page', sanitize_text_field($_POST['custom_404_page']));
            echo '<div class="notice notice-success"><p>404 settings saved successfully!</p></div>';
        }

        $custom_404 = get_option('custom_404_page');
        ?>
        <div class="wrap">
            <h1>404 Redirect Settings</h1>
            
            <form method="post">
                <?php wp_nonce_field('404_settings_nonce'); ?>
                <table class="form-table">
                    <tr>
                        <th scope="row">Custom 404 Page Path</th>
                        <td>
                            <input type="text" name="custom_404_page" value="<?php echo esc_attr($custom_404); ?>" class="regular-text">
                            <p class="description">Enter the path of 404 Page</p>
                        </td>
                    </tr>
                </table>

                <input type="submit" name="save_404_settings" class="button button-primary" value="Save Settings">
            </form>

            <h2>404 Redirect Logs</h2>
            <?php $this->render_logs_table('404'); ?>
        </div>
        <?php
    }

    public function render_301_page() {
        if (isset($_POST['save_redirects']) && check_admin_referer('301_redirects_nonce')) {
            $redirect_rules = array();
            $old_urls = isset($_POST['redirect_rules']['old']) ? $_POST['redirect_rules']['old'] : array();
            $new_urls = isset($_POST['redirect_rules']['new']) ? $_POST['redirect_rules']['new'] : array();
            
            foreach ($old_urls as $index => $old_url) {
                if (!empty($old_url) && !empty($new_urls[$index])) {
                    $redirect_rules[sanitize_text_field($old_url)] = sanitize_text_field($new_urls[$index]);
                }
            }
            
            update_option('redirect_rules', $redirect_rules);
            echo '<div class="notice notice-success"><p>301 redirects saved successfully!</p></div>';
        }

        $redirect_rules = get_option('redirect_rules', array());
        ?>
        <div class="wrap">
            <h1>301 Redirect Settings</h1>
            
            <form method="post">
                <?php wp_nonce_field('301_redirects_nonce'); ?>
                
                <table class="widefat" id="redirect-rules">
                    <thead>
                        <tr>
                            <th>Old URL</th>
                            <th>New URL</th>
                            <th>Action</th>
                        </tr>
                    </thead>
                    <tbody>
                        <?php if (!empty($redirect_rules)): ?>
                            <?php foreach ($redirect_rules as $old => $new): ?>
                            <tr>
                                <td>
                                    <input type="text" name="redirect_rules[old][]" 
                                           value="<?php echo esc_attr($old); ?>" 
                                           class="regular-text" placeholder="/old-page">
                                </td>
                                <td>
                                    <input type="text" name="redirect_rules[new][]" 
                                           value="<?php echo esc_attr($new); ?>" 
                                           class="regular-text" placeholder="/new-page">
                                </td>
                                <td>
                                    <button type="button" class="button remove-redirect">Remove</button>
                                </td>
                            </tr>
                            <?php endforeach; ?>
                        <?php endif; ?>
                        <tr>
                            <td colspan="3">
                                <button type="button" class="button add-redirect">Add New Redirect</button>
                            </td>
                        </tr>
                    </tbody>
                </table>

                <p class="submit">
                    <input type="submit" name="save_redirects" class="button button-primary" value="Save Redirects">
                </p>
            </form>

            <h2>301 Redirect Logs</h2>
            <?php $this->render_logs_table('301'); ?>
        </div>

        <script>
        jQuery(document).ready(function($) {
            $('.add-redirect').click(function() {
                var row = '<tr>' +
                    '<td><input type="text" name="redirect_rules[old][]" class="regular-text" placeholder="/old-page"></td>' +
                    '<td><input type="text" name="redirect_rules[new][]" class="regular-text" placeholder="/new-page"></td>' +
                    '<td><button type="button" class="button remove-redirect">Remove</button></td>' +
                    '</tr>';
                $(this).closest('tr').before(row);
            });

            $(document).on('click', '.remove-redirect', function() {
                $(this).closest('tr').remove();
            });
        });
        </script>
        <?php
    }

    private function render_logs_table($type) {
        global $wpdb;
        $logs = $wpdb->get_results($wpdb->prepare(
            "SELECT * FROM $this->logs_table WHERE redirect_type = %s ORDER BY access_date DESC LIMIT 100",
            $type
        ));
        ?>
        <table class="widefat">
            <thead>
                <tr>
                    <th>Source URL</th>
                    <th>Destination URL</th>
                    <th>Date</th>
                    <th>IP Address</th>
                </tr>
            </thead>
            <tbody>
                <?php foreach ($logs as $log): ?>
                <tr>
                    <td><?php echo esc_html($log->source_url); ?></td>
                    <td><?php echo esc_html($log->destination_url); ?></td>
                    <td><?php echo esc_html($log->access_date); ?></td>
                    <td><?php echo esc_html($log->ip_address); ?></td>
                </tr>
                <?php endforeach; ?>
            </tbody>
        </table>
        <?php
    }
}

// Initialize the plugin
new RedirectSuite();